'use strict';

const pageResults = require('graph-results-pager');

const { Promise } = require('bluebird')

const { graphAPIEndpoints } = require('./constants')

const innai = require('./queries/innai');
const blocks = require('./queries/blocks');
const charts = require('./queries/charts');
const exchange = require('./queries/exchange');
const exchange_v1 = require('./queries/exchange_v1')
const mastermerchant = require('./queries/mastermerchant');
const innovaegenerator = require('./queries/innovaegenerator')
const innovaispawner = require('./queries/innovaispawner')
const timelock =  require('./queries/timelock');
const lockup = require('./queries/lockup');
const forum = require('./queries/forum');
const utils = require('./utils');

module.exports = {
	pageResults,
	graphAPIEndpoints,
	innai,
	blocks,
	charts,
	exchange,
	exchange_v1,
	mastermerchant,
	innovaegenerator,
	innovaispawner,
	timelock,
	lockup,
	forum,
	utils,
	async timeseries({blocks = undefined, timestamps = undefined, target = undefined} = {}, targetArguments) {
		if(!target) { throw new Error("genova-data: Target function undefined"); }
		if(!blocks && !timestamps) { throw new Error("genova-data: Timeframe undefined"); }

		if(blocks) {
			return Promise.map(blocks, async (block) => ({
				block,
				data: await target({block, ...targetArguments})
			}));
		}

		else {
			return Promise.map(timestamps, async (timestamp) => ({
				timestamp,
				data: await target({timestamp, ...targetArguments})
			}));
		}
	},
};
