const pageResults = require('graph-results-pager')

const { request, gql } = require('graphql-request')

const { priceUSD: innaiPriceUSD } = require('./innai')
const { token: tokenInfo } = require('./exchange')
const { ethPrice: ethPriceUSD } = require('./exchange')
const { info: masterMerchantInfo } = require('./mastermerchant')
const { pool: merchantPool } = require('./mastermerchant')
const { pools: merchantPools } = require('./mastermerchant')

// accessed by chainId
const ENDPOINTS = {
  1: 'https://api.thegraph.com/subgraphs/name/luckyswap/forum',
  250: 'https://api.thegraph.com/subgraphs/name/luckyswap/fantom-forum',
  56: 'https://api.thegraph.com/subgraphs/name/luckyswap/bsc-forum',
  137: 'https://api.thegraph.com/subgraphs/name/luckyswap/matic-forum',
  100: 'https://api.thegraph.com/subgraphs/name/luckyswap/xdai-forum',
}

const MASTER_CONTRACT = '0x2cba6ab6574646badc84f0544d05059e57a5dc42'

module.exports = {
  async clones({ masterAddress = undefined, chainId = undefined } = {}) {
    if(!masterAddress) { throw new Error("genova-data: Master Address undefined"); }
    if(!chainId) { throw new Error("genova-data: Chain Id undefined"); }

    return pageResults({
      api: ENDPOINTS[chainId],
      query: {
        entity: 'clones',
        selection: {
          where: {
            masterContract: `\\"${masterAddress.toLowerCase()}\\"`
          }
        },
        properties: clones.properties
      }
    })
      .then(results => clones.callback(results))
      .catch(err => console.log(err));
  },

  async venetianStakedInfo() {
    const results = await pageResults({
      api: ENDPOINTS[1],
      query: {
        entity: 'venetianPairs',
        properties: venetianStakedInfo.properties
      }
    })

    let result = {}
    result.innaiUSD = await innaiPriceUSD();
    result.ethUSD = await ethPriceUSD();

    let masterMerchant = await masterMerchantInfo();
    result.totalAP = masterMerchant.totalAllocPoint;
    result.innaiPerBlock = masterMerchant.innaiPerBlock;

    let pools = await merchantPools();
    let onsen_pools = pools.map(pool => pool.pair)
    let filtered_results = results.filter(pair => onsen_pools.includes(pair.id))
    result.venetianPairs = filtered_results

    return venetianStakedInfo.callback(result)
  },

}

const clones = {
  properties: [
    'id',
    'data'
  ],

  callback(results) {
    return results.map(({ id, data, block, timestamp }) => ({
      address: id,
      data: data
    }));
  }
}

const venetianStakedInfo = {
  properties: [
    'id',
    'name',
    'symbol',
    'asset { id, symbol, decimals }',
    'collateral { id, symbol, decimals }',
    'totalAssetBase',
    'totalAssetElastic'
  ],

  async callback(results) {
    return await Promise.all(results.venetianPairs.map(async (result) => {
      let asset = await tokenInfo({ token_address: result.asset.id });
      let assetPool = await merchantPool({ pool_address: result.id });
      if (assetPool === undefined) { return }
      let stakedAmt = assetPool.innlpBalance * 1e18;
      let balanceUSD = (stakedAmt * asset.derivedETH * results.ethUSD) / (10 ** result.asset.decimals);
      let rewardPerBlock = ((1 / results.totalAP) * results.innaiPerBlock);
      let roiPerBlock = (rewardPerBlock * results.innaiUSD) / balanceUSD;
      let roiPerYear = roiPerBlock * 6500 * 365

      return {
        id: result.id,
        name: result.name,
        symbol: result.symbol,
        asset: result.asset.id,
        assetSymbol: result.asset.symbol,
        assetDecimals: Number(result.asset.decimals),
        collateral: result.collateral.id,
        collateralSymbol: result.collateral.symbol,
        collateralDecimals: Number(result.collateral.decimals),
        totalAssetBase: Number(result.totalAssetBase),
        totalAssetElastic: Number(result.totalAssetElastic),
        totalAssetStaked: Number(stakedAmt),
        assetDecimals: Number(result.asset.decimals),
        balanceUSD: balanceUSD,
        rewardPerBlock: rewardPerBlock,
        roiPerBlock: roiPerBlock,
        roiPerYear: roiPerYear
      }
    }));
  }
}
