const ws = require('isomorphic-ws');
const { SubscriptionClient } = require('subscriptions-transport-ws'); 

const { request, gql } = require('graphql-request');

const { graphAPIEndpoints, graphWSEndpoints, innovaegeneratorAddress } = require('./../constants')
const { timestampToBlock } = require('./../utils');

module.exports = {
    async info({block = undefined, timestamp = undefined} = {}) {
        block = block ? block : timestamp ? (await timestampToBlock(timestamp)) : undefined;
        block = block ? `block: { number: ${block} }` : "";

        const result = await request(graphAPIEndpoints.innovaegenerator,
            gql`{
                    innovaegenerator(id: "${innovaegeneratorAddress}", ${block}) {
                        ${info.properties.toString()}
                    }
                }`
        );

        return info.callback(result.innovaegenerator);
    },

    observeInfo() {
        const query = gql`
            subscription {
                innovaegenerator(id: "${innovaegeneratorAddress}") {
                    ${info.properties.toString()}
                }
        }`;

        const client = new SubscriptionClient(graphWSEndpoints.innovaegenerator, { reconnect: true, }, ws,);
        const observable = client.request({ query });

        return {
            subscribe({next, error, complete}) {
                return observable.subscribe({
                    next(results) {
                        next(info.callback(results.data.innovaegenerator));
                    },
                    error,
                    complete
                });
            }
        };
    },

    async user({block = undefined, timestamp = undefined, user_address = undefined} = {}) {
        if(!user_address) { throw new Error("genova-data: User address undefined"); }

        block = block ? block : timestamp ? (await timestampToBlock(timestamp)) : undefined;
        block = block ? `block: { number: ${block} }` : "";

        const result = await request(graphAPIEndpoints.innovaegenerator,
            gql`{
                    user(id: "${user_address.toLowerCase()}", ${block}) {
                        ${user.properties.toString()}
                    }
                }`
        );

        return user.callback(result.user);
    },
}

const info = {
    properties: [
        'decimals',
        'name',
        'innai',
        'symbol',
        'totalSupply',
        'ratio',
        'InnovaeMinted',
        'InnovaeBurned',
        'innaiStaked',
        'innaiStakedUSD',
        'innaiHarvested',
        'innaiHarvestedUSD',
        'InnovaeAge',
        'InnovaeAgeDestroyed',
        'updatedAt'
    ],

    callback(results) {
        return ({
            decimals: Number(results.decimals),
            name: results.name,
            innai: results.innai,
            symbol: results.symbol,
            totalSupply: Number(results.totalSupply),
            ratio: Number(results.ratio),
            InnovaeMinted: Number(results.InnovaeMinted),
            InnovaeBurned: Number(results.InnovaeBurned),
            innaiStaked: Number(results.totalSupply) * Number(results.ratio),
            innaiStakedUSD: Number(results.innaiStakedUSD),
            innaiHarvested: Number(results.innaiHarvested),
            innaiHarvestedUSD: Number(results.innaiHarvestedUSD),
            InnovaeAge: Number(results.InnovaeAge),
            InnovaeAgeDestroyed: Number(results.InnovaeAgeDestroyed),
            updatedAt: Number(results.updatedAt)
        })
    }
};

const user = {
    properties: [
        'Innovae',
        'InnovaeIn',
        'InnovaeOut',
        'InnovaeMinted',
        'InnovaeBurned',
        'InnovaeOffset',
        'InnovaeAge',
        'InnovaeAgeDestroyed',
        'innaiStaked',
        'innaiStakedUSD',
        'innaiHarvested',
        'innaiHarvestedUSD',
        'innaiIn',
        'innaiOut',
        'usdOut',
        'usdIn',
        'updatedAt',
        'innaiOffset',
        'usdOffset'
    ],

    callback(results) {
        return ({
            Innovae: Number(results.Innovae),
            InnovaeIn: Number(results.InnovaeIn),
            InnovaeOut: Number(results.InnovaeOut),
            InnovaeMinted: Number(results.InnovaeMinted),
            InnovaeBurned: Number(results.InnovaeBurned),
            InnovaeOffset: Number(results.InnovaeOffset),
            InnovaeAge: Number(results.InnovaeAge),
            InnovaeAgeDestroyed: Number(results.InnovaeAgeDestroyed),
            innaiStaked: Number(results.innaiStaked),
            innaiStakedUSD: Number(results.innaiStakedUSD),
            innaiHarvested: Number(results.innaiHarvested),
            innaiHarvestedUSD: Number(results.innaiHarvestedUSD),
            innaiIn: Number(results.innaiIn),
            innaiOut: Number(results.innaiOut),
            usdOut: Number(results.usdOut),
            usdIn: Number(results.usdIn),
            updatedAt: Number(results.updatedAt),
            innaiOffset: Number(results.innaiOffset),
            usdOffset: Number(results.usdOffset)
        })
    }
};